use crate::model::{logbook::Logbook, stat::Stat, user::User};
use serde::Serialize;

#[derive(Serialize, PartialEq, Debug)]
pub(crate) enum Level {
    NONE,
    BRONZE,
    SILVER,
    GOLD,
    DIAMOND,
    DONE,
}

impl Level {
    fn required_km(&self) -> i32 {
        match self {
            Level::BRONZE => 40000,
            Level::SILVER => 80000,
            Level::GOLD => 100000,
            Level::DIAMOND => 200000,
            Level::DONE => 0,
            Level::NONE => 0,
        }
    }

    fn next_level(km: i32) -> Self {
        if km < Level::BRONZE.required_km() {
            Level::BRONZE
        } else if km < Level::SILVER.required_km() {
            Level::SILVER
        } else if km < Level::GOLD.required_km() {
            Level::GOLD
        } else if km < Level::DIAMOND.required_km() {
            Level::DIAMOND
        } else {
            Level::DONE
        }
    }

    pub(crate) fn curr_level(km: i32) -> Self {
        if km < Level::BRONZE.required_km() {
            Level::NONE
        } else if km < Level::SILVER.required_km() {
            Level::BRONZE
        } else if km < Level::GOLD.required_km() {
            Level::SILVER
        } else if km < Level::DIAMOND.required_km() {
            Level::GOLD
        } else {
            Level::DIAMOND
        }
    }

    pub(crate) fn desc(&self) -> &str {
        match self {
            Level::BRONZE => "Bronze",
            Level::SILVER => "Silber",
            Level::GOLD => "Gold",
            Level::DIAMOND => "Diamant",
            Level::DONE => "",
            Level::NONE => "-",
        }
    }
}

#[derive(Serialize)]
pub(crate) struct Next {
    level: Level,
    desc: String,
    missing_km: i32,
    required_km: i32,
    rowed_km: i32,
}

impl Next {
    pub(crate) fn new(rowed_km: i32) -> Self {
        let level = Level::next_level(rowed_km);
        let required_km = level.required_km();
        let missing_km = required_km - rowed_km;
        Self {
            desc: level.desc().to_string(),
            level,
            missing_km,
            required_km,
            rowed_km,
        }
    }
}

pub(crate) async fn new_level_with_last_log(
    db: &mut sqlx::Transaction<'_, sqlx::Sqlite>,
    user: &User,
) -> Option<String> {
    let rowed_km = Stat::total_km_tx(db, user).await.rowed_km;

    if let Some(last_logbookentry) = Logbook::completed_with_user_tx(db, user).await.last() {
        let last_trip_km = last_logbookentry.logbook.distance_in_km.unwrap();
        if Level::curr_level(rowed_km) != Level::curr_level(rowed_km - last_trip_km as i32) {
            return Some(Level::curr_level(rowed_km).desc().to_string());
        }
    }

    None
}
